xquery version "3.1";

module namespace bibref="https://mollenhauer-edition.de/ns/conversions/bibref";

(:~
 : Convert Module Bibref
 : *******************************************************************************************
 : This module contains functions to create bibliographical data for KMG-Texts.
 :
 : @version 1.0 (2023-01-09)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace selection="https://mollenhauer-edition.de/ns/conversions/selection" at "selection.xqm";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace html="http://www.w3.org/1999/xhtml";

(:~
 : This function returns a tei:author which child tei:name which contains tei:surname and tei:forename.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $author tei:author
 : @return tei:author
 :)
declare function bibref:make-author($author as element(tei:author)) as element(tei:author)
{
    element tei:author {
        element tei:name {
            element tei:surname {
                $author/tei:name/tei:surname/text()
            },
            ', ',
            element tei:forename {
                $author/tei:name/tei:forename/text()
            }
        }
    }
};

(:~
 : This function produces the URL for one specific version of a KMG-text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $teiHeader tei:teiHeader
 : @param $versionToken xs:string
 : @return tei:idno
 :)
declare function bibref:make-url($teiHeader as element(tei:teiHeader), $versionToken as xs:string) as element(tei:idno)
{
    let $baseURL := 'https://mollenhauer-edition.de/'
    return
        if ($versionToken = 'tia') then
            element tei:idno {
                attribute type { 'URL' },
                $baseURL || 'kmg.html?file=' || tokenize($teiHeader//tei:idno[@type='URI']/text(), ':')[2] || '&amp;edition=tia'
            }
        else
            element tei:idno {
                attribute type { 'URL' },
                $baseURL || 'kmg.html?file=' || tokenize($teiHeader//tei:idno[@type='URI']/text(), ':')[2] || '&amp;edition=' || $versionToken
            }
};

(:declare function bibref:make-url($teiHeader as element(tei:teiHeader), $versionToken as xs:string) as element(tei:idno):)
(:{:)
(:    let $baseURL := 'https://mollenhauer-edition.de/works/':)
(:    return:)
(:        if ($versionToken = 'tia') then:)
(:            element tei:idno {:)
(:                attribute type { 'URL' },:)
(:                $baseURL || $teiHeader//tei:idno[@type='KMG']/text() || '-tia':)
(:            }:)
(:        else:)
(:            element tei:idno {:)
(:                attribute type { 'URL' },:)
(:                $baseURL || $teiHeader//tei:idno[@type='KMG']/text() || '-' || $versionToken:)
(:            }:)
(:};:)

(:~
 : This function produces the version description for one specific version of a KMG-text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $teiHeader tei:teiHeader
 : @param $versionToken xs:string
 : @return xs:string
 :)
declare function bibref:make-version-desc($teiHeader as element(tei:teiHeader), $versionToken as xs:string) as xs:string
{
    if ($versionToken = 'tia') then
        let $firstYearData := selection:get-zotero-data(tokenize($teiHeader//tei:sourceDesc/tei:listWit/tei:witness[1]/tei:listWit/tei:witness[1]/tei:bibl[1]/@corresp, ':')[2])
        let $firstYear := (
            if ($firstYearData//fn:string[@key="date"]/text()) then
                $firstYearData//fn:string[@key="date"]/text()
            else
                $firstYearData/@zoteroID || ': ' || $firstYearData/text()
            )
        let $lastYearData := selection:get-zotero-data(tokenize($teiHeader//tei:sourceDesc/tei:listWit/tei:witness[last()]/tei:listWit/tei:witness[last()]/tei:bibl[last()]/@corresp, ":")[2])
        let $lastYear := (
            if ($lastYearData//fn:string[@key="date"]/text()) then
                $lastYearData//fn:string[@key="date"]/text()
            else
                $lastYearData/@zoteroID || ': ' || $lastYearData/text()
            )
        return
            'textkritisch-interaktive Ansicht ' || $firstYear || '-' || $lastYear
    else if ($versionToken = 'sta') then
        let $yearData := (
            if ($teiHeader//tei:textClass/tei:keywords/tei:term[@key='documenttype:da6784cb-a922-4ddf-aca1-dc96b24f07d4']) then
                selection:get-zotero-data(tokenize($teiHeader//tei:sourceDesc/tei:listWit/tei:witness[last()]/tei:listWit/tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
            else
                selection:get-zotero-data(tokenize($teiHeader//tei:sourceDesc/tei:listWit/tei:witness[1]/tei:listWit/tei:witness[1]/tei:bibl[1]/@corresp, ':')[2])
            )
        let $year := (
            if ($yearData//fn:string[@key="date"]/text()) then
                $yearData//fn:string[@key="date"]/text()
            else
                $yearData/@zoteroID || ': ' || $yearData/text()
            )
        return
            $year
    else
        $teiHeader//tei:sourceDesc/tei:listWit/tei:witness[@xml:id = $versionToken]/tei:desc/text()
};

(:~
 : This function produces a tei:biblStruct which contain bibliographical data of one specific version of a KMG-text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc TEI-Document
 : @param $versionToken xs:string
 : @return tei:biblStruct
 :)
declare function bibref:make-biblstruct($doc as node(), $versionToken as xs:string) as element()
{
    let $teiHeader := $doc//tei:teiHeader
    let $authors := (
        for $author in $teiHeader//tei:titleStmt/tei:author
        return
            bibref:make-author($author)
        )
    let $edition := element tei:idno {
        attribute type { 'editionnumber' },
        $teiHeader//tei:idno[@type='KMG']/text() || '-' || $versionToken
    }
    let $title := element tei:title {
        attribute type { 'main' },
        $teiHeader//tei:titleStmt/tei:title[@type="main"]/string()
    }
    let $subtitle  := element tei:title {
        attribute type { 'sub' },
        $teiHeader//tei:titleStmt/tei:title[@type="sub"]/string()
    }
    let $versionDesc := element tei:title {
        attribute type { 'versiondesc' },
        bibref:make-version-desc($teiHeader, $versionToken)
    }
    let $seriesSubTitle := (
        if ($versionToken = 'sta') then
            element tei:title {
                    attribute type { 'sub' },
                    'Studienausgabe'
                }
        else
            element tei:title {
                    attribute type { 'sub' },
                    'Historisch-kritische Edition'
                }
        )
    let $volumePagesURL := (
        if ($versionToken = 'sta') then
             element tei:idno {
                attribute type { 'pages' },
                'Bd. X, S. Y–Z'
            }
        else
            bibref:make-url($teiHeader, $versionToken)
        )
    let $imprint := (
        if ($versionToken = 'sta') then
            <tei:imprint>
                <tei:pubPlace>Göttingen</tei:pubPlace>
                <tei:publisher>Universitätsverlag</tei:publisher>
                <tei:date>2024</tei:date>
            </tei:imprint>
        else
            <tei:imprint>
                <tei:date>2024</tei:date>
            </tei:imprint>
        )
    return
        if ($versionToken = 'tia' or $versionToken = 'sta' or $teiHeader//tei:sourceDesc/tei:listWit/tei:witness[@xml:id = $versionToken])  then
            element tei:biblStruct {
                element tei:analytic {
                    $authors,
                    $edition,
                    $title,
                    $subtitle,
                    $versionDesc
                },
                element tei:monogr {
                    <tei:title type='main'>Klaus Mollenhauer Gesamtausgabe</tei:title>,
                    $seriesSubTitle,
                    <tei:editor>
                        <tei:name>
                            <tei:surname>Dietrich</tei:surname>, <tei:forename>Cornelie</tei:forename>
                        </tei:name>
                    </tei:editor>,
                    <tei:editor>
                        <tei:name>
                            <tei:surname>Horn</tei:surname>, <tei:forename>Klaus-Peter</tei:forename>
                        </tei:name>
                    </tei:editor>,
                    <tei:editor>
                        <tei:name>
                            <tei:surname>Müller</tei:surname>, <tei:forename>Hans-Rüdiger</tei:forename>
                        </tei:name>
                    </tei:editor>,
                    $volumePagesURL,
                    $imprint
                }
            }
        else
            element tei:bibl {
                'Fassungskürzel ' || $versionToken || ' existiert nicht.'
            }
};

(:~
 : This function produces a list of tei:listBibl which contain bibliographical data of KMG-texts.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $IDs Numbers of the edition (and optionally version token)
 : @return tei:listBibl
 :)
declare function bibref:make-listbibl($IDs as xs:string*) as element(tei:listBibl)
{
    element tei:listBibl {
        for $ID in $IDs
        let $doc := doc($config:sade-data || selection:TGURI-from-KMG-number(substring($ID, 1, 3)) || '.xml')
        let $versionTokens := selection:dissolve-ID($ID, $doc)
        return
            for $versionToken in $versionTokens
            return
                bibref:make-biblstruct($doc, $versionToken)
  }
};

(:~
 : This function produces formatted authors.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $analytic tei:analytic
 : @return xs:string
 :)
declare function bibref:format-authors($analytic as element(tei:analytic)) as xs:string
{
    let $authors := $analytic/tei:author
    return
        if (count($authors) > 2) then
            let $number := count($authors)
            return
                let $formattedAuthors := (
                for $author at $pos in $authors
                return
                if ($pos = $number) then
                    $author
                else if ($pos = ($number - 1)) then
                    $author/string() || ' &amp; '
                else
                    $author/string() || ', '
                )
                return
                    string-join($formattedAuthors)
        else if (count($authors) = 2) then
            $authors[1]/string() || ' &amp; ' || $authors[2]/string()
        else
            $authors/string()
};

(:~
 : This function produces formatted titles.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $analytic tei:analytic
 : @return xs:string
 :)
declare function bibref:format-title($analytic as element(tei:analytic)) as xs:string
{
    if ($analytic/tei:title[@type="sub"]) then
        string-join(($analytic/tei:title[@type="main"], '. ', $analytic/tei:title[@type="sub"], ' (', $analytic/tei:title[@type="versiondesc"], ').'))
    else
        string-join(($analytic/tei:title[@type="main"], '(', $analytic/tei:title[@type="versiondesc"], ').'))
};

(:~
 : This function converts a tei:listBibl to html:ul which contains formatted bibliographical data for KMG-texts.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $listBibl tei:listBibl
 : @return html:ul
 :)
declare function bibref:make-HTML($listBibl as element(tei:listBibl)) as element(html:ul)
{
    element html:ul {
        for $biblStruct in $listBibl/tei:biblStruct
        let $authors := bibref:format-authors($biblStruct/tei:analytic)
        let $editionnumber := string-join(
            (
                '(',
                $biblStruct//tei:idno[@type = 'editionnumber'],
                ').'
            )
        )
        let $title := bibref:format-title($biblStruct/tei:analytic)
        let $kmg := element html:i {
            string-join(
                (
                    $biblStruct/tei:monogr/tei:title[@type = 'main'],
                    '. ',
                    $biblStruct/tei:monogr/tei:title[@type = 'sub']
                )
            )
        }
        let $year := string-join(
            (
                ' (',
                $biblStruct/tei:monogr//tei:date,
                ').'
            )
        )
        let $editors := string-join(
            (
                $biblStruct/tei:monogr/tei:editor[1]//tei:forename,
                ' ',
                $biblStruct/tei:monogr/tei:editor[1]//tei:surname,
                ', ',
                $biblStruct/tei:monogr/tei:editor[2]//tei:forename,
                ' ',
                $biblStruct/tei:monogr/tei:editor[2]//tei:surname,
                ' &amp; ',
                $biblStruct/tei:monogr/tei:editor[3]//tei:forename,
                ' ',
                $biblStruct/tei:monogr/tei:editor[3]//tei:surname,
                '. '
            )

        )
        let $volumePagesURL := (
            if ($biblStruct/tei:monogr/tei:idno[@type = 'pages']) then
                $biblStruct/tei:monogr/tei:idno/text()
            else
                element html:a {
                    attribute href { $biblStruct/tei:monogr/tei:idno/string() },
                    $biblStruct/tei:monogr/tei:idno/string()
                },
            '.'
        )
        let $publisherPlace := (
            if ($biblStruct/tei:monogr//tei:pubPlace) then
                (
                    string-join(
                        (
                            $biblStruct/tei:monogr//tei:pubPlace,
                            ': ',
                            $biblStruct/tei:monogr//tei:publisher
                        )
                    )
                )
            else
                ()
        )
        return
            element html:li {
                $authors,
                $editionnumber,
                $title,
                'In ',
                $kmg,
                $year,
                $editors,
                $volumePagesURL,
                $publisherPlace
            }
    }
};