xquery version "3.1";

module namespace citations="https://mollenhauer-edition.de/ns/conversions/citations";

(:~
 : Convert Module Citations
 : *******************************************************************************************
 : This module contains functions to retrieve bibliographical data from zotero for bibliographical references made by Klaus Mollenhauer.
 :
 : @version 1.1 (2023-02-16)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)

import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "selection.xqm";
import module namespace simplify="https://mollenhauer-edition.de/ns/conversions/simplify" at "simplify.xqm";

declare namespace tei="http://www.tei-c.org/ns/1.0";

(:~
 : This function produces data for all creators (editors or authors) of one title in JSON-format.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $creators-array fn:array@key='creators'
 : @return data as JSON
 :)

declare function citations:get-creators($creators-array as node()) {
    if ($creators-array//fn:string[@key="creatorType"]/text() = "author") then
        for $creator in $creators-array/fn:map
        return
            if ($creator/fn:string[@key="creatorType"]/text() = "author") then
                let $lastname := (
                    if ($creator/fn:string[@key="lastName"]) then
                        $creator/fn:string[@key="lastName"]
                    else if ($creator/fn:string[@key="name"]) then
                        $creator/fn:string[@key="name"]
                    else
                        ()
                    )
                return
                    map {
                        "lastName" : $lastname/string(),
                        "firstName" : $creator/fn:string[@key="firstName"]/string(),
                        "role" : $creator/fn:string[@key="creatorType"]/string()
                    }
            else
                ()
    else
        for $creator in $creators-array/fn:map
        return
            if ($creator/fn:string[@key="creatorType"]/text() = "editor") then
                let $lastname := (
                    if ($creator/fn:string[@key="lastName"]) then
                        $creator/fn:string[@key="lastName"]
                    else if ($creator/fn:string[@key="name"]) then
                        $creator/fn:string[@key="name"]
                    else
                        ()
                    )
                return
                    map {
                        "lastName" : $lastname/string(),
                        "firstName" : $creator/fn:string[@key="firstName"]/string(),
                        "role" : $creator/fn:string[@key="creatorType"]/string()
                    }
            else
                ()
};

(:~
 : This function retrieves the Zotero-ID.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node Element-node with attribute @ref
 : @return The $zotero-Id(s) as string (one or more items)
 :)

declare function citations:get-zotero-id($node as node())
{
    let $zoteros := tokenize($node/@corresp, " ")
    return
        for $zotero in $zoteros
        let $zotero-Id := tokenize($zotero, ':')[2]
        return
            $zotero-Id
};


(:~
 : This function produces bibliographical data for one title in JSON-format. If the data cannot be retrieved from Zotero, a similar JSON will be produced consisting of the zotero-ID and the error message.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $zotero-Id ID of the Zotero-Item
 : @param $bibliograpy true if this title is cited within a bibliography, false if not
 : @param $indirect-citation true if this title is cited within a quote, false if not
 : @param $kmg-bibl true if this title is cited with a KMG-bibl, false if not
 : @return bibliographical data as JSON
 :)

declare function citations:title-data($zotero-Id as xs:string, $bibliography as xs:boolean, $indirect-citation as xs:boolean, $kmg-bibl as xs:boolean)
{
    let $zotero-data := selection:get-zotero-data($zotero-Id)
    return
        try {
            let $publication-year := $zotero-data//fn:string[@key="date"]/text()
            let $title := $zotero-data//fn:string[@key="title"]/text()
            let $creators := $zotero-data//fn:array[@key="creators"]

            let $citations := $zotero-data//fn:string[@key="citation"]/text()
            let $creators := citations:get-creators($creators)
                return
                    map {
                        "citation" : $citations,
                        "persons" : array {
                                    $creators
                                },
                        "title" : $title,
                        "types" : map {
                            "inBibliography" : $bibliography,
                            "indirectCitation" : $indirect-citation,
                            "kmgBibl" : $kmg-bibl
                        },
                        "year" : xs:integer($publication-year),
                        "zoteroID" : $zotero-Id
                    }
        } catch * {
            map {
                    "citation" : (),
                    "persons" : (),
                    "title" : (),
                    "types" : (),
                    "year" : (),
                    "zoteroID" : $zotero-Id,
                    "errorMessage" : $zotero-data/text(),
                    "errorNumber" : xs:integer($zotero-data/@n)
                }
        }
};

(:~
 : This function retrieves the data bibliographical data for citations.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $bibls tei:bibl
 : @return bibliographical data as JSON
 :)
declare function citations:all-citations($bibls as node()*)
{
    for $bibl in $bibls
        let $zotero-Ids := citations:get-zotero-id($bibl)
        return
            for $zotero-Id in $zotero-Ids
            return
                if (boolean($zotero-Id)) then
                    if ($bibl/ancestor::tei:quote) then
                        if ($bibl/ancestor::tei:div[@type="literature"]) then
                            if ($bibl/@type='KMG') then
                                citations:title-data($zotero-Id, true(), true(), true())
                            else
                                citations:title-data($zotero-Id, true(), true(), false())
                        else
                            citations:title-data($zotero-Id, false(), true(), false())
                    else if ($bibl/ancestor::tei:div[@type="literature"]) then
                        if ($bibl/@type='KMG') then
                            citations:title-data($zotero-Id, true(), false(), true())
                        else
                            citations:title-data($zotero-Id, true(), false(), false())
                    else if ($bibl/@type='KMG') then
                        citations:title-data($zotero-Id, false(), false(), true())
                    else
                        citations:title-data($zotero-Id, false(), false(), false())
                else
                    map {
                        "citation" : (),
                        "persons" : (),
                        "title" : (),
                        "types" : (),
                        "year" : (),
                        "zotero-id" : $zotero-Id,
                        "error-message" : 'Zotero-ID does not exist',
                        "error-number" : "3"
                    }
};

(:~
 : This function retrieves the bibliographical data for all citations, except for citations in commentaries and in tei:teiHeader.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc A TEI-Document
 : @param $fassung The Fassung that should be used
 : @return bibliographical data as JSON
 :)
declare function citations:all-citations-single-version($doc as node()*, $fassung as xs:string)
{
    let $version := simplify:all($doc, $fassung)
    let $bibls := $version//tei:text[@type='edition']//tei:bibl
    return
        citations:all-citations($bibls)
};

(:~
 : This function retrieves the bibliographical data for all citations in a given text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node A TEI-encoded text
 : @param $fassung The Fassung that should be used
 : @return bibliographical data as JSON
 :)
declare function citations:all-citations-monography($node as node()*, $fassung as xs:string)
{
    let $version := simplify:all($node, $fassung)
    let $bibls := $version//tei:bibl
    return
        citations:all-citations($bibls)
};