xquery version "3.1";

module namespace json="https://mollenhauer-edition.de/ns/conversions/json";

(:~
 : Convert Module JSON
 : *******************************************************************************************
 : This module contains functions to produce JSON-output.
 :
 : @version 2.1 (2023-02-16)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)

import module namespace tei2plain = "https://mollenhauer-edition.de/ns/conversions/tei2plain" at "tei2plain.xqm";
import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "selection.xqm";
import module namespace citations="https://mollenhauer-edition.de/ns/conversions/citations" at "citations.xqm";
import module namespace persnames="https://mollenhauer-edition.de/ns/conversions/persnames" at "persnames.xqm";
import module namespace simplify="https://mollenhauer-edition.de/ns/conversions/simplify" at "simplify.xqm";

import module namespace functx="http://www.functx.com";

declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace tei="http://www.tei-c.org/ns/1.0";

declare option output:method "json";
declare option output:indent "yes";

(:~
 : This formats the author names as (a sequence of) xs:string.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc The TEI-file
 : @return (Sequence of) xs:string
 :)
declare function json:author-names($doc as node()) as xs:string*
{
    for $author in $doc//tei:author
    let $names := string-join(
            (
                $author//tei:surname,
                ', ',
                $author//tei:forename
            )
        )
    return
        $names
};

(:~
 : This function produces JSON-output of bibliographical data within quotations (made by Klaus Mollenhauer) from one document.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc The TEI-file
 : @param $fassungen The Fassungen which should be converted
 :)

declare function json:citations-single($doc as node(), $fassungen as xs:string+)
{
    let $work-zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
    return
        let $firstpublication := (
            if ($work-zotero-data/name() = 'error') then
                $work-zotero-data/@zoteroID || ': ' || $work-zotero-data/text()
            else
                $work-zotero-data//fn:string[@key="date"]/text()
            )
        return
            map {
                "metadata" : map {
                    "author" : json:author-names($doc),
                    "title" : fn:normalize-space($doc//tei:titleStmt/tei:title[@type = 'main']),
                    "type" : "bibls",
                    "year_first_publication" : xs:integer($firstpublication)
                },
                "versions" : array {
                    for $fassung in $fassungen
                    let $fassung-zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[@xml:id=$fassung]//tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
                    return
                        let $year := (
                        if ($fassung-zotero-data/name() = 'error') then
                            $fassung-zotero-data/@zoteroID || ': ' || $fassung-zotero-data/text()
                        else
                            $fassung-zotero-data//fn:string[@key="date"]/text()
                        )
                    return
                        if (selection:check-fassung($doc, $fassung)) then
                            map {
                                "metadata" : map {
                                    "version" : concat('Fassung ', $fassung),
                                    "year" : xs:integer($year)
                                },
                                "data" : array {citations:all-citations-single-version($doc, $fassung)}
                            }
                        else ()
                }
            }
};

(:~
 : This function produces JSON-output of bibliographical data within quotations (made by Klaus Mollenhauer) for a series of documents.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $path Directory, where the TEI-files are stored
 : @param $filenames Names of the TEI-files
 : @param $fassungen The Fassungen which should be converted
 :)

declare function json:citations($path as xs:string, $filename as xs:string, $fassungen as xs:string*) {
    let $doc := fn:doc(concat($path, '/', $filename))
    return
        if (exists($fassungen)) then
            let $validfassungen := (
                for $fassung in $fassungen
                return
                    if (selection:check-fassung($doc, $fassung)) then
                        $fassung
                    else
                        ()
            )
            return
                if (count($validfassungen) > 0) then
                    json:citations-single($doc, $validfassungen)
            else ()
        else
            let $wits := $doc//tei:sourceDesc/tei:listWit/tei:witness/@xml:id
            return
                json:citations-single($doc, $wits)
};

(:~
 : This function produces JSON-output of data for persons mentioned by Klaus Mollenhauer for one document.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc The TEI-file
 : @param $fassungen The Fassungen which should be converted
 :)

declare function json:persnames-single($doc as node(), $fassungen as xs:string+)
{
    let $work-zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
    return
        let $firstpublication := (
            if ($work-zotero-data/name() = 'error') then
                $work-zotero-data/@zoteroID || ': ' || $work-zotero-data/text()
            else
                $work-zotero-data//fn:string[@key="date"]/text()
            )
        return
            map {
                "metadata" : map {
                    "author" : json:author-names($doc),
                    "title" : fn:normalize-space($doc//tei:titleStmt/tei:title[@type = 'main']),
                    "type" : "persnames",
                    "year_first_publication" : xs:integer($firstpublication)
                },
                "versions" : array {
                    for $fassung in $fassungen
                    let $fassung-zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[@xml:id=$fassung]//tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
                    return
                        let $year := (
                        if ($fassung-zotero-data/name() = 'error') then
                            $fassung-zotero-data/@zoteroID || ': ' || $fassung-zotero-data/text()
                        else
                            $fassung-zotero-data//fn:string[@key="date"]/text()
                        )
                    return
                        if (selection:check-fassung($doc, $fassung)) then
                            map {
                                "metadata" : map {
                                    "version" : concat('Fassung ', $fassung),
                                    "year" : xs:integer($year)
                                },
                                "data" : array {persnames:all-persnames-single-version($doc, $fassung)}
                            }
                        else ()
                }
            }
};

(:~
 : This function produces JSON-output of data for persons mentioned by Klaus Mollenhauer for a series of documents.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $path Directory, where the TEI-files are stored
 : @param $filenames Names of the TEI-files
 : @param $fassungen The Fassungen which should be converted
 :)

declare function json:persnames($path as xs:string, $filename as xs:string, $fassungen as xs:string*) {
    let $doc := fn:doc(concat($path, '/', $filename))
    return
        if (exists($fassungen)) then
            let $validfassungen := (
                for $fassung in $fassungen
                return
                    if (selection:check-fassung($doc, $fassung)) then
                        $fassung
                    else
                        ()
            )
            return
                if (count($validfassungen) > 0) then
                    json:persnames-single($doc, $validfassungen)
            else ()
        else
            let $wits := $doc//tei:sourceDesc/tei:listWit/tei:witness/@xml:id
            return
                json:persnames-single($doc, $wits)
};

(:~
 : This function produces JSON-output of transformations from TEI to plain text for one document.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc The TEI-file
 : @param $fassungen The Fassungen which should be converted
 :)

declare function json:plain-text-single($doc as node(), $fassungen as xs:string+) {
    let $work-zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
    return
        let $firstpublication := (
            if ($work-zotero-data/name() = 'error') then
                $work-zotero-data/@zoteroID || ': ' || $work-zotero-data/text()
            else
                $work-zotero-data//fn:string[@key="date"]/text()
            )
        return
            map {
                "metadata" : map {
                    "author" : json:author-names($doc),
                    "title" : fn:normalize-space($doc//tei:titleStmt/tei:title[@type = 'main']),
                    "type" : "plaintext",
                    "year_first_publication" : xs:integer($firstpublication)
                },
                "versions" : array {
                    for $fassung in $fassungen
                    let $fassung-zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[@xml:id=$fassung]//tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
                    return
                        let $year := (
                        if ($fassung-zotero-data/name() = 'error') then
                            $fassung-zotero-data/@zoteroID || ': ' || $fassung-zotero-data/text()
                        else
                            $fassung-zotero-data//fn:string[@key="date"]/text()
                        )
                    return
                        if (selection:check-fassung($doc, $fassung)) then
                            map {
                                "metadata" : map {
                                    "version" : concat('Fassung ', $fassung),
                                    "year" : xs:integer($year)
                                },
                                "data" : fn:string-join(tei2plain:complete-corr($doc, $fassung), ' ')
                            }
                        else ()
                }
            }
};

(:~
 : This function produces JSON-output of transformations from TEI to plain text for a series of documents.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $path Directory, where the TEI-files are stored
 : @param $filename Name of the TEI-file
 : @param $fassungen The Fassungen which should be converted
 :)

declare function json:plain-text($path as xs:string, $filename as xs:string, $fassungen as xs:string*) {
    let $doc := fn:doc(concat($path, '/', $filename))
    return
        if (exists($fassungen)) then
            let $validfassungen := (
                for $fassung in $fassungen
                return
                    if (selection:check-fassung($doc, $fassung)) then
                        $fassung
                    else
                        ()
            )
            return
                if (count($validfassungen) > 0) then
                    json:plain-text-single($doc, $validfassungen)
            else ()
        else
            let $wits := $doc//tei:sourceDesc/tei:listWit/tei:witness/@xml:id
            return
                json:plain-text-single($doc, $wits)
};

(:~
 : This function produces JSON-output of data for persons mentioned by Klaus Mollenhauer, bibliographical data within quotations (made by Klaus Mollenhauer) or of the plain text for every chapter.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc The TEI-file
 : @param $fassungen The Fassungen which should be converted
 : @param $type Type of requested data
 :)
declare function json:analyze-monography-single($doc as node(), $fassungen as xs:string+, $type as xs:string) {
    let $work-zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
    return
        let $firstpublication := (
            if ($work-zotero-data/name() = 'error') then
                $work-zotero-data/@zoteroID || ': ' || $work-zotero-data/text()
            else
                $work-zotero-data//fn:string[@key="date"]/text()
            )
        let $divs := $doc//tei:text[@type='edition']/tei:body//tei:div[@type='chapter']
        let $lastfassung := data($doc//tei:sourceDesc/tei:listWit/tei:witness[last()]/@xml:id)
        return
            for $div at $pos in $divs
            let $divSingle := simplify:all($div, $lastfassung)
            let $chapterTitle := normalize-space($divSingle/descendant::tei:head[1])
            return
                map {
                    "metadata" : map {
                        "author" : json:author-names($doc),
                        "title" : $pos || ': ' || $chapterTitle,
                        "type" : $type,
                        "year_first_publication" : xs:integer($firstpublication)
                    },
                    "versions" : array {
                        for $fassung in $fassungen
                        let $fassung-zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[@xml:id=$fassung]//tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
                        return
                            let $year := (
                            if ($fassung-zotero-data/name() = 'error') then
                                $fassung-zotero-data/@zoteroID || ': ' || $fassung-zotero-data/text()
                            else
                                $fassung-zotero-data//fn:string[@key="date"]/text()
                            )
                        return
                            if (selection:check-fassung($doc, $fassung)) then
                                let $data := (
                                    if ($type = 'citations') then
                                        if ($div/ancestor::tei:*[@wit]) then
                                            if(contains($div/ancestor::tei:*/@wit, $fassung)) then
                                                array {citations:all-citations-monography($div, $fassung)}
                                            else
                                                []
                                        else
                                            array {citations:all-citations-monography($div, $fassung)}
                                    else if ($type = 'persnames') then
                                        if ($div/ancestor::tei:*[@wit]) then
                                            if(contains($div/ancestor::tei:*/@wit, $fassung)) then
                                                array {persnames:all-persnames-monography($div, $fassung)}
                                            else
                                                []
                                        else
                                            array {persnames:all-persnames-monography($div, $fassung)}
                                    else if ($type = 'plaintext') then
                                        if ($div/ancestor::tei:*[@wit]) then
                                            if(contains($div/ancestor::tei:*/@wit, $fassung)) then
                                                fn:string-join(tei2plain:complete-corr($div, $fassung), ' ')
                                            else
                                                ''
                                        else
                                            fn:string-join(tei2plain:complete-corr($div, $fassung), ' ')
                                    else
                                        ()
                                    )
                                return
                                    map {
                                        "metadata" : map {
                                            "version" : concat('Fassung ', $fassung),
                                            "year" : xs:integer($year)
                                        },
                                        "data" : $data
                                    }
                            else ()
                    }
                }
};

(:~
 : This function produces JSON-output of data for persons mentioned by Klaus Mollenhauer, bibliographical data within quotations (made by Klaus Mollenhauer) or of the plain text for a certain TEI-file.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $path Directory, where the TEI-files are stored
 : @param $filename Name of the TEI-file
 : @param $fassungen The Fassungen which should be converted
 : @param $type Type of requested data
 :)
declare function json:analyze-monography($path as xs:string, $filename as xs:string, $fassungen as xs:string*, $type as xs:string) {
    let $doc := fn:doc(concat($path, '/', $filename))
    return
        if (exists($fassungen)) then
            let $validfassungen := (
                for $fassung in $fassungen
                return
                    if (selection:check-fassung($doc, $fassung)) then
                        $fassung
                    else
                        ()
            )
            return
                if (count($validfassungen) > 0) then
                    (
                        json:analyze-monography-single($doc, $validfassungen, $type),
                        if ($type = 'citations') then
                            json:citations-single($doc, $validfassungen)
                        else if ($type = 'persnames') then
                            json:persnames-single($doc, $validfassungen)
                        else if ($type = 'plaintext') then
                            json:plain-text-single($doc, $validfassungen)
                        else
                            ()
                    )
            else ()
        else
            let $wits := $doc//tei:sourceDesc/tei:listWit/tei:witness/@xml:id
            return
                (
                    json:analyze-monography-single($doc, $wits, $type),
                    if ($type = 'citations') then
                        json:citations-single($doc, $wits)
                    else if ($type = 'persnames') then
                        json:persnames-single($doc, $wits)
                    else if ($type = 'plaintext') then
                        json:plain-text-single($doc, $wits)
                    else
                        ()
                )
};