xquery version "3.1";

module namespace persnames="https://mollenhauer-edition.de/ns/conversions/persnames";

(:~
 : Convert Module Persnames
 : *******************************************************************************************
 : This module contains functions to retrievedata for persons mentioned by Klaus Mollenhauer.
 :
 : @version 1.1 (2023-02-16)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)

import module namespace simplify="https://mollenhauer-edition.de/ns/conversions/simplify" at "simplify.xqm";

import module namespace http = "http://expath.org/ns/http-client";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace pnd="http://textgrid.info/namespaces/vocabularies/pnd";

(:~
 : This function retrieves the GND-ID.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node Element-node with attribute @ref
 : @return The $gnd-Id(s) as string (one or more items)
 :)

declare function persnames:get-GND-ID($node as node())
{
    let $refs := tokenize($node/@ref)
    return
        for $ref in $refs
        return
            if (starts-with($ref, 'gnd:')) then
                let $gndID := tokenize($ref, ":")[2]
                return
                    $gndID
            else if (starts-with($ref, '#')) then
                let $linkelement := $node/root()//*[@xml:id = data(replace($ref, '#', ''))][1]
                let $gndID := tokenize($linkelement/@ref, ":")[2]
                return
                    $gndID
            else
                ()
};

(:~
 : This function requests the GND-data of one GND-item.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $gnd-Id ID of the Zotero-Item
 : @return The $gnd-data of the requested $gnd-Item
 :)

declare function persnames:get-GND-data($gnd-Id as xs:string)
{
    let $api-url := "https://ref.de.dariah.eu/pndsearch/pndquery.xql?id=" || $gnd-Id
    return
        let $response := (
            let $requestedData := http:send-request(<http:request href="{$api-url}" method="get"/>)
            let $response-head := $requestedData[1]
            return (
                $response-head,
                if ( $response-head/@status = 200 ) then
                    if ( $requestedData[2]//pnd:entry/@id = $gnd-Id ) then
                        $requestedData[2]
                    else
                        ( $requestedData[2],
                        element error {
                            attribute n {2},
                            attribute GNDID {$gnd-Id},
                            'GND-data could not be fetched for unknown reason (GND-ID might not exist).'
                        } )
                else (
                    $requestedData[2],
                    element error {
                        attribute n {1},
                        attribute GNDID {$gnd-Id},
                        concat('GND-data could not be fetched because http:request failed with status code ', $response-head/@status)
                    }
                )
            )
        )
        return
            if ( count($response) > 2 )
            then ( $response[3] )
            else ( $response[2] )
};

(:~
 : This function produces GND-data for one person in JSON-format. If the data cannot be retrieved from the GND, a similar JSON will be produced consisting of the GND-ID and the error message.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $gnd-Id ID of the Zotero-Item
 : @param $bibliograpy true if this person is mentioned within a bibliography, false if not
 : @param $indirect-citation true if this person is mentioned within a quote, false if not
 : @return bibliographical data as JSON
 :)

declare function persnames:persname-data($gnd-Id as xs:string, $bibliography as xs:boolean, $indirect-citation as xs:boolean)
{
    let $gnd-data := persnames:get-GND-data($gnd-Id)
    return
        if ($gnd-data/name() = 'error') then
            map {
                    "errorMessage" : $gnd-data/text(),
                    "errorNumber" : $gnd-data/@n,
                    "GNDID" : $gnd-Id,
                    "name" : (),
                    "types" : ()
                }
        else
            map {
            "GNDID" : $gnd-Id,
            "name" : $gnd-data//pnd:si[1]/text(),
            "types" : map {
                    "inBibliography" : $bibliography,
                    "indirectCitation" : $indirect-citation
                }
        }
};

(:~
 : This function retrieves data for persons referenced with tei:name and tei:rs.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $persnames tei:rs or tei:name
 : @return GND-data as JSON
 :)
declare function persnames:all-persnames($persnames as node()*)
{
    for $persname in $persnames
        let $gnd-Ids := persnames:get-GND-ID($persname)
        return
            for $gnd-Id in $gnd-Ids
            return
                if (boolean($gnd-Id)) then
                    if ($persname/ancestor::tei:quote) then
                        if ($persname/ancestor::tei:div[@type="literature"]) then
                            persnames:persname-data($gnd-Id, true(), true())
                        else
                            persnames:persname-data($gnd-Id, false(), true())
                    else if ($persname/ancestor::tei:div[@type="literature"]) then
                        persnames:persname-data($gnd-Id, true(), false())
                    else
                        persnames:persname-data($gnd-Id, false(), false())
                else
                    map {
                        "errorMessage" : 'GND-ID does not exist',
                        "errorNumber" : "3",
                        "GNDID" : $gnd-Id,
                        "name" : (),
                        "types" : ()
                    }
};

(:~
 : This function retrieves data for persons referenced with tei:name and tei:rs, except for the elements in commentaries and in tei:teiHeader.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc A TEI-Document
 : @param $fassung The Fassung that should be used
 : @return GND-data as JSON
 :)
declare function persnames:all-persnames-single-version($doc as node()*, $fassung as xs:string)
{
    let $version := simplify:all($doc, $fassung)
    let $persnames := $version//tei:text[@type='edition']//tei:*[@type="person"]
    return
        persnames:all-persnames($persnames)
};

(:~
 : This function retrieves data for persons referenced with tei:name and tei:rs in a given text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node A TEI-encoded text
 : @param $fassung The Fassung that should be used
 : @return GND-data as JSON
 :)
declare function persnames:all-persnames-monography($node as node()*, $fassung as xs:string)
{
    let $version := simplify:all($node, $fassung)
    let $persnames := $version//tei:*[@type="person"]
    return
        persnames:all-persnames($persnames)
};