xquery version "3.1";

(:~
 : This module provides the API endpoint for data for distant reading.
 :
 : @version 1.0 (2023-02-16)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)
module namespace rest-analyses="https://mollenhauer-edition.de/ns/rest-analyses";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "./selection.xqm";
import module namespace json = "https://mollenhauer-edition.de/ns/conversions/json" at "./json.xqm";
import module namespace rest="http://exquery.org/ns/restxq";

declare namespace output ="http://www.w3.org/2010/xslt-xquery-serialization";

(: ~
 : This functions allows to request bibliographical data within quotations (made by Klaus Mollenhauer) as a JSON object.
 :
 : @param $title Title(s) of the texts
 : @param $year Year(s) in which the texts has been published the first time
 : @param $fassungen Fassung(en) which are requested
 : @return A JSON object
 :)
declare
    %rest:path("api/citations")
    %rest:GET
    %rest:form-param("title", "{$title}")
    %rest:form-param("year", "{$year}")
    %rest:form-param("fassungen", "{$fassungen}")
    %output:media-type("application/json")
    %output:method("json")
function rest-analyses:citations($title as xs:string*, $year as xs:string*, $fassungen as xs:string*)
as item()*
{
    let $filenames := selection:getFileNames($config:sade-data, $title, $year)
    return
        array {
        for $filename in $filenames
            return
                try {
                    map {
                        "error": $filename/@zoteroID || ': ' || $filename/text()
                    }
                } catch * {
                    json:citations($config:sade-data, $filename, $fassungen)
                }
        }
};

(: ~
 : This functions allows to request data for persons mentioned by Klaus Mollenhauer as a JSON object.
 :
 : @param $title Title(s) of the texts
 : @param $year Year(s) in which the texts has been published the first time
 : @param $fassungen Fassung(en) which are requested
 : @return A JSON object
 :)
declare
    %rest:path("api/persons")
    %rest:GET
    %rest:form-param("title", "{$title}")
    %rest:form-param("year", "{$year}")
    %rest:form-param("fassungen", "{$fassungen}")
    %output:media-type("application/json")
    %output:method("json")
function rest-analyses:persnames($title as xs:string*, $year as xs:string*, $fassungen as xs:string*)
as item()*
{
    let $filenames := selection:getFileNames($config:sade-data, $title, $year)
    return
        array {
        for $filename in $filenames
            return
                try {
                    map {
                        "error": $filename/@zoteroID || ': ' || $filename/text()
                    }
                } catch * {
                    json:persnames($config:sade-data, $filename, $fassungen)
                }
        }
};

(: ~
 : This functions allows to request the plain text of certain TEI-files as a JSON object.
 :
 : @param $title Title(s) of the texts
 : @param $year Year(s) in which the texts has been published the first time
 : @param $fassungen Fassung(en) which are requested
 : @return A JSON object
 :)
declare
    %rest:path("api/plaintext")
    %rest:GET
    %rest:form-param("title", "{$title}")
    %rest:form-param("year", "{$year}")
    %rest:form-param("fassungen", "{$fassungen}")
    %output:media-type("application/json")
    %output:method("json")
function rest-analyses:plain-text($title as xs:string*, $year as xs:string*, $fassungen as xs:string*)
as item()*
{
    let $filenames := selection:getFileNames($config:sade-data, $title, $year)
    return
        array {
        for $filename in $filenames
            return
                try {
                    map {
                        "error": $filename/@zoteroID || ': ' || $filename/text()
                    }
                } catch * {
                    json:plain-text($config:sade-data, $filename, $fassungen)
                }
        }
};

(: ~
 : This functions allows to request the plain text, the citations or the persname-references of certain TEI-files as a JSON object.
 :
 : @param $title Title(s) of the texts
 : @param $fassungen Fassung(en) which are requested
 : @param $type Type of requested data
 : @return A JSON object
 :)
declare
    %rest:path("api/monography")
    %rest:GET
    %rest:form-param("title", "{$title}")
    %rest:form-param("fassungen", "{$fassungen}")
    %rest:form-param("type", "{$type}")
    %output:media-type("application/json")
    %output:method("json")
function rest-analyses:monography($title as xs:string*, $fassungen as xs:string*, $type as xs:string*)
as item()*
{
    let $filenames := selection:getFileNames($config:sade-data, $title, ())
    return
        array {
        for $filename in $filenames
            return
                try {
                    map {
                        "error": $filename/@zoteroID || ': ' || $filename/text()
                    }
                } catch * {
                    json:analyze-monography($config:sade-data, $filename, $fassungen, $type)
                }
        }
};