xquery version "3.1";

module namespace selection="https://mollenhauer-edition.de/ns/conversions/selection";

(:~
 : Convert Module Selection
 : *******************************************************************************************
 : This module contains functions to select certain XML-files.
 :
 : @version 2.0 (2023-02-15)
 : @status stable
 : @author Max-Ferdinand Zeterberg
 :)

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";

import module namespace http = "http://expath.org/ns/http-client";
import module namespace functx="http://www.functx.com";

declare namespace tei = "http://www.tei-c.org/ns/1.0";

(:~
 : This function checks if the TEI-Document has the same title as $title.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc: TEI-Document
 : @param $title: Exact title of the Text --> //tei:titleStmt/tei:title[@type='main']
 : @return True or False
 :)

declare function selection:checkTitle($doc as node(), $title as xs:string*)
{
    contains(lower-case(normalize-space($doc//tei:titleStmt/tei:title[@type = 'main'])), lower-case(normalize-space($title)))
};

(:~
 : This function requests the zotero-data of one zotero-item.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $zotero-Id ID of the Zotero-Item
 : @return The Zotero-data of the requested Zotero-Item formatted as TEI
 :)

declare function selection:get-zotero-data($zotero-Id as xs:string)
{
    let $api-url := "https://mollenhauer-edition.de/api/zotero/items/" || $zotero-Id
    return
        let $response := (
            let $requestedData := http:send-request(<http:request href="{$api-url}" method="get"/>)
            let $response-head := $requestedData[1]
            return (
                $response-head,
                if ( $response-head/@status = 200 )
                then (
                    try {
                    $requestedData[2] => util:binary-to-string() => json-to-xml()
                    } catch * {
                        $requestedData[2],
                        element error {
                            attribute n {2},
                            attribute zoteroID {$zotero-Id},
                            'Zotero-data could not be fetched because there was a major error in binary conversion.'
                        }
                    }
                )
                else (
                    $requestedData[2],
                    element error {
                        attribute n {1},
                        attribute zoteroID {$zotero-Id},
                        concat('Zotero-data could not be fetched because http:request failed with status code ', $response-head/@status)
                    }
                )
            )
        )
        return
            if ( count($response) > 2 )
            then ( $response[3] )
            else ( $response[2] )
};

(:~
 : This function returns the filename or the filenames:
 :  --> If only one file exist that matches title, year, or title and year, then only one filename will be returned.
 :  --> If more than one file matches title, year, or title and year, then more than one filename will be returned.
 :  --> If no file matches title, year, or title and year, then an empty sequence will be returned.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $path directory where the file is stored
 : @param $titles Exact titles of the Text --> //tei:titleStmt/tei:title[@type='main']
 : @param $years Years in which the Text was published first
 : @return A sequence with either one, more or zero items --> filename(s)
 :)

declare function selection:getFileNames($path as xs:string, $titles as xs:string*, $years as xs:string*)
{
    if (exists($titles)) then
        for $title in $titles
        return
            if (exists($years)) then
                    for $doc in collection($path)
                    let $zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
                    return
                        if ($zotero-data/name() = 'error') then
                            $zotero-data
                        else
                            let $titleYear := $zotero-data//fn:string[@key="date"]/text()
                            where (functx:is-value-in-sequence($titleYear, $years) and selection:checkTitle($doc, $title))
                            return
                                util:document-name($doc)
            else
                for $doc in collection($path)
                where (selection:checkTitle($doc, $title))
                return
                    util:document-name($doc)
    else if (exists($years)) then
            for $doc in collection($path)
            let $zotero-data := selection:get-zotero-data(tokenize($doc//tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:bibl[1]/@corresp, ":")[2])
            return
                if ($zotero-data/name() = 'error') then
                    $zotero-data
                else
                    let $titleYear := $zotero-data//fn:string[@key="date"]/text()
                    where (functx:is-value-in-sequence($titleYear, $years))
                    return
                        util:document-name($doc)
    else
        for $doc in collection($path)
        return
            util:document-name($doc)
};

(:~
 : This function checks if the Value of $fassung exists as tei:witness.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc TEI-Document
 : @param $fassung Fassung of the KMG-Text -->
 : //tei:sourceDesc[1]/tei:listWit[1]/tei:witness[@*[namespace-uri() = 'http://www.w3.org/XML/1998/namespace' and
 :             local-name() = 'id']
 : @return True or False
 :)
declare function selection:check-fassung($doc as node()*, $fassung as xs:string)
{
    exists($doc//tei:sourceDesc[1]/tei:listWit[1]/tei:witness[@*[namespace-uri() =
               'http://www.w3.org/XML/1998/namespace' and local-name() = 'id'] = $fassung])
};

(:~
 : This function returns version tokens.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $ID Number of the edition (and optionally version token)
 : @return xs:string or sequence of xs:string
 :)
declare function selection:dissolve-ID($ID as xs:string, $doc as node()) as xs:string+
{

        if (contains($ID, '-')) then
            let $version := tokenize($ID, '-')[2]
            return
                $version
        else
            let $versions := data($doc//tei:sourceDesc/tei:listWit/tei:witness/@xml:id)
            return
                functx:value-union($versions, ('tia', 'sta'))
};

(:~
 : This function retrieves the TextGrid-URI of a document.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $kmgnumber Number of the KMG-text as xs:string
 : @return xs:string
 :)
declare function selection:TGURI-from-KMG-number($kmgnumber as xs:string) as xs:string
{
    for $doc in collection($config:sade-data)
    where $doc//tei:idno[@type='KMG']/text() = $kmgnumber
    return
        substring-after($doc//tei:idno[@type='URI']/text(), 'textgrid:')
};
