xquery version "3.1";

(:~
 : This module contains the transformation and complementary functions of the
 : interactive view for the KMG website.
 :
 : @author Simon Sendler
 : @version 1.0
 : @since 1.0
 : @see http://bdn-edition.de/ns/digital
 : TODO: cleanup namespaces and imports
 : TODO: update namespaces to something project-specific
 :)
module namespace transformKMG="http://mollenhauer-edition.de/ns/transform";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace functx="http://www.functx.com";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace html="http://www.w3.org/1999/xhtml";
declare namespace tg="http://textgrid.info/namespaces/metadata/core/2010";
declare default element namespace "http://www.w3.org/1999/xhtml";

(:~
 : The main transformation routine. It iterates all nodes of a given TEI-XML
 : document and attempts to initiate transformation.
 : Should transformation fail, a span-Element will be included in the return,
 : including the error's code, description, as well as the node that raised
 : the exception.
 :
 : @param $nodes The nodes of a given TEI/XML document
 : @param $filename e.g. "2228v.xml"
 : @param $mode The edition view being transformed. "Digital" for interactive
 : view, for single view, the xml:id of the respective witness
 : @return The respective HTML nodes
 :)
declare function transformKMG:transform(
    $nodes as node()*, $filename as xs:string, $mode as xs:string)
as node()*
{
    for $node in $nodes
    return
        try {
            if ($node/@xml:id) then
                element span {
                    attribute class { "text-anchor" },
                    attribute id { $node/@xml:id }
                }
            else
                (),
            if ( $node/@copyOf ) then
                local:copy-node($node, $filename, $mode)
            else
                local:main($node, $filename, $mode)
        } catch * {
            util:log-system-out(util:system-time() || " " || $filename || ": Error during transformation for " || $mode || "."),
            element span {
                attribute class {"error-message"},
                "Error: ", $err:code, ". ", $err:description,
                element br {},
                "Offending node: ", functx:path-to-node-with-pos($node)
            }
        }
};

(:~
 : The main transformation routine. It takes a TEI-xml node and transforms it
 : to a HTML node.
 :
 : For easier findability, elements in the typeswitch are sorted alphabetically.
 :
 : @param $nodes A TEI-xml node
 : @param $filename A TG-URI, e.g. "2228v.xml"
 : @param $mode The edition view being transformed.
 : @return the respective HTML nodes
 :)
declare function local:main(
    $node as node(), $filename as xs:string, $mode as xs:string)
{
    typeswitch ($node)

        case text() return
            text { $node }

        case comment() return
            ()

        case element (tei:app) return
            if ( $mode = "digital" ) then
                if ( $node/tei:lem/tei:note ) then
                    transformKMG:transform($node/tei:lem/tei:note, $filename, $mode)
                else if ( $node/tei:lem/tei:seg[@type= "margin" ] ) then
                    transformKMG:transform($node/tei:lem/tei:seg[@type= "margin" ], $filename, $mode)
                else
                    transformKMG:make-interactive-apparatus($node, $filename, $mode)
            else
                transformKMG:transform($node/child::*[contains(@wit, $mode)], $filename, $mode)

        case element (tei:bibl) return
            (transformKMG:transform($node/node(), $filename, $mode),
            for $token in $node/tokenize(@corresp, " ") return
                element span {
                    attribute data-target {substring-after($token, ":")},
                    attribute class
                    {
                        "editorial-icon js-editorial-info--bibl" ||
                        (if ($node/@type = "KMG") then
                            "-KMG"
                        else
                            ())
                    },
                    text { "&#8288;" },
                    element i {
                        if ($node/@type = "KMG") then
                            (attribute class {"fas fa-book-medical"})
                        else
                            (attribute class {"fas fa-book"})
                    }
                })

        case element (tei:cell)
        return
            element td
            {
                if ( $node/@rows ) then
                    attribute rowspan { $node/@rows }
                else
                    (),
                if ( $node/@cols ) then
                    attribute colspan { $node/@cols }
                else
                    (),
                transformKMG:transform($node/node(), $filename, $mode)
            }

        case element (tei:choice)
        return
            transformKMG:choice($node, $filename, $mode)

        case element (tei:corr) return
            if ($node/node()) then
                transformKMG:transform($node/node(), $filename, $mode)
            else
                text {"&#xF8;"}

        case element (tei:div) return
            element div {
                attribute {"type"} {$node/@type},
                transformKMG:transform($node/node(), $filename, $mode)
            }

        case element (tei:figure) return
            element figure {
                attribute class {"image"},
                transformKMG:transform($node/node(), $filename, $mode)
            }

        case element (tei:front) return
            element div {
                attribute class {"preface"},
                transformKMG:transform($node/node(), $filename, $mode)
            }

        case element (tei:gap) return
            if ( $node/@unit = "characters" ) then
                element i {
                    attribute class { "fas fa-ellipsis-h" }
                }
            else
                element p {
                    element i {
                        attribute class { "fas fa-ellipsis-h" }
                    }
                }

        case element (tei:graphic) return
            if (fn:exists($node/@url) and fn:not(fn:empty($node/@url))) then (
                let $target := replace($node/@url, "textgrid:", "/db/data/sade/textgrid/img/") || ".jpg"
                let $text := 
                    if (fn:exists($node/ancestor::tei:figure//tei:figDesc/text())) then (
                        fn:normalize-space(fn:string-join($node/ancestor::tei:figure//tei:figDesc/text()))
                    )
                    else ("")
                return
                    element span {
                        if (fn:exists($node/@corresp)) then (
                            attribute data-target {$node/@corresp/string()},
                            attribute class {"js-editorial-icon--art"}
                        )
                        else (),
                        element img {
                            attribute src {$target},
                            attribute alt {$text},
                            attribute title {$text}
                        }
                    }
            )
            else ()
        
        case element (tei:figDesc) return
            ()
            
        case element (tei:head) return
            if ($node/@type="ToC") then
                ()
            else if ($node/parent::tei:div/@type = "part") then
                element h2 {
                    transformKMG:transform($node/node(), $filename, $mode)
                }
            else if ($node/parent::tei:div/@type = "chapter") then
                element h3 {
                    transformKMG:transform($node/node(), $filename, $mode)
                }
            else if ($node/@type="sub") then
                element h3 {
                    attribute class {"heading--sub-heading"},
                    transformKMG:transform($node/node(), $filename, $mode)
                }
            else if ($node/parent::tei:div/@type = "section") then
                element h4 {
                    transformKMG:transform($node/node(), $filename, $mode)
                }
            else if ( $node/parent::tei:table ) then
                element span {
                    attribute class { "table__heading" },
                    transformKMG:transform($node/node(), $filename, $mode)
                }
            else
                element h5 {
                transformKMG:transform($node/node(), $filename, $mode)
                }

        case element (tei:hi) return
            element span {
                attribute class { 
                    if (fn:exists($node/@rend)) then
                        $node/@rend/string()
                    else if (fn:exists($node/ancestor::tei:text/@id)) then 
                        transformKMG:getHiRendStyleFromHeader($filename,$node/ancestor::tei:text/@id/string())
                        else ()
                },
                transformKMG:transform($node/node(), $filename, $mode)
            }

        case element ( tei:item ) return
            transformKMG:make-list-item($node, $filename, $mode)

        case element ( tei:join ) return
            let $join := transformKMG:resolve-join($node)
            return
                transformKMG:transform($join, $filename, $mode)

        case element (tei:l) return
            element span {
                attribute class {"line-group__line"},
                transformKMG:transform($node/node(), $filename, $mode),
                if ($node/following-sibling::tei:l) then
                    element br {}
                else
                    ()
            }

        case element (tei:label) return
            switch ($node/@type)
                case "head" return
                    element span {
                        attribute class {"heading__label"},
                        transformKMG:transform($node/node(), $filename, $mode)
                    }
                case "list" return
                    ()
                case "note_anchor" return
                    element sup {
                        attribute class {"authorial-note__anchor"},
                        transformKMG:transform($node/node(), $filename, $mode),
                        " "
                    }
                default return
                    transformKMG:transform($node/node(), $filename, $mode)

        case element (tei:lb) return
            element br {}

        case element (tei:lem) return
            if ( $mode = "digital" ) then
                transformKMG:make-interactive-lemma($node, $filename, $mode)
            else
                transformKMG:transform($node/node(), $filename, $mode)

        case element (tei:lg) return
            element div {
                attribute class {"line-group"},
                if ( $node/@rend ) then attribute class { $node/@rend } else (),
                transformKMG:transform($node/node(), $filename, $mode)
            }

        case element (tei:list) return
            element ul {
                (if ($node/parent::tei:item) then
                  attribute class { "list list--nested" || $node/@rend }
                else if ($node/ancestor::tei:rdg) then
                  attribute class {"list list--rdg" || $node/@rend }
                else
                  attribute class {"list" || $node/@rend }
                ),
                transformKMG:transform($node/node(), $filename, $mode)
            }

        case element ( tei:media )
        return
            transformKMG:media($node)

        case element (tei:milestone) return
            if ($node/@type = "paragraph") then
                element br {}
            else
                ()

        case element (tei:name) return
            let $type := $node/@type
            let $classMap :=
                map {
                    "art": "artname",
                    "brandname": "brandname",
                    "music": "musicname",
                    "org": "orgname",
                    "periodica": "periodname",
                    "person": "persname"
                }
            return
                element span {
                    attribute class {"name " || $classMap($type) || " editorial-icon js-editorial-info--" || $type},
                    if ($node/@ref) then
                        attribute data-target {
                            if (starts-with($node/@ref, "#")) then
                                transformKMG:resolve-reference($node) => replace("((gnd|textgrid)?:)+", "")
                            else
                                replace($node/@ref, "((gnd|textgrid)?:)+", "")
                        }
                    else(),
                    if ( $node/@type = "music" ) then
                        attribute data-key
                        {
                            if ( $node/@key ) then
                                $node/@key
                            else
                                $node//text()
                        }
                    else
                        (),
                    transformKMG:transform($node/node(), $filename, $mode),
                    text { "&#8288;" }
                }

        case element (tei:note) return
            if ($node/@type="commentary" or $node/@type="editorial") then
                let $id := util:hash(generate-id($node), "md5")
                let $target := "#" || $id
                return
                    (text { "&#8288;" },
                    element span {
                        attribute class { "editorial-icon js-editorial-info--note" },
                        attribute data-target {$id},
                        element i {
                            attribute class {"fas fa-comment-alt"}
                        }
                    },
                    element div {
                        attribute id {$id},
                        attribute class {"modal-body__content"},
                        transformKMG:transform($node/node(), $filename, $mode)
                    }
                    )
            else if ($node/ancestor::tei:p and not($mode = "digital" and $node/parent::tei:rdg) or $node/ancestor::tei:table ) then
                    (text { "&#8288;" },
                    element sup {
                        attribute class {"authorial-note__anchor"},
                        if ($node//tei:app//tei:label[@type="note_anchor"]) then
                            if ( $mode = "digital" ) then
                                transformKMG:transform($node//tei:lem[descendant::tei:label[@type="note_anchor"]]//tei:label[@type="note_anchor"], $filename, $mode)
                            else
                                transformKMG:transform($node//tei:app/*[contains(@wit, $mode)][descendant::tei:label[@type="note_anchor"]]//tei:label[@type="note_anchor"], $filename, $mode)
                        else
                            $node//tei:label[@type="note_anchor"]
                    })
            else
                element div {
                    attribute class {"authorial-note"},
                    transformKMG:transform($node/node(), $filename, $mode)
                }

        case element (tei:p) return
            (if ($node/ancestor::tei:figure) then
                element figcaption {
                    transformKMG:transform($node/node(), $filename, $mode)
                }
            else
                element div {
                    attribute class {
                        if ($node/parent::tei:item/ancestor::tei:list) then
                            "paragraph paragraph--list"
                        else if ($node/@rend = "small_letters") then
                            "paragraph small_letters"
                        else if ($node/@rend) then
                            "paragraph " || $node/@rend
                        else
                            "paragraph"
                    },
                    if (fn:exists($node/@n)) then
                        element span {
                            attribute class {"editorial"},
                            if (fn:number($node/@n)) then
                                "[" || $node/@n || "] "
                            else
                                "[" || "ERROR: wrong format of paragraph number" || "] "
                        }
                    else (),
                    transformKMG:transform($node/node(), $filename, $mode)
                },
            for $note in $node//tei:note[not(@type)] return
                if ( $note/parent::tei:rdg and not(contains($note/parent::tei:rdg/@wit, $mode)) or $note/ancestor::tei:table ) then
                    ()
                else
                    element div {
                        attribute class {"authorial-note"},
                        transformKMG:transform($note/node(), $filename, $mode)
                    }
            )

        case element (tei:pb) return
            if ($mode = "digital") then
                element span {
                attribute class {"editorial"},
                    "|" || replace($node/@edRef, "#", "") || " " || $node/@n || "|"
                }
            else
                let $wits := replace($node/@edRef, "#", "") => tokenize()
                return
                    if (contains($wits, $mode)) then
                        element span {
                            attribute class {"pb"},
                            "|" || replace($node/@edRef, "#", "") || " " || $node/@n || "|"
                        }
                    else
                    ()

        case element (tei:q) return
            transformKMG:make-quote($node, $filename, $mode)

        case element (tei:quote) return
            transformKMG:make-quote($node, $filename, $mode)

        case element (tei:ref) return
            element span {
                attribute class {"reference"},
                transformKMG:transform($node/node(), $filename, $mode),
                text { "&#8288;" },
                for $token in $node/tokenize(@target, " ") return
                    element a {
                        attribute class {"link--text"},
                        attribute href {
                            if (starts-with($token, "#")) then
                                $token
                            else
                                replace($token, "textgrid:", "/exist/apps/sade/textgrid/data/kmg/")
                        },
                        element i {
                            attribute class {"fas fa-link"}
                        }
                    }
            }

        case element ( tei:row )
        return
            element tr {
                transformKMG:transform($node/node(), $filename, $mode)
            }

        case element (tei:seg) return
            let $id := util:hash(generate-id($node/tei:sic), "md5")
            let $target := "#" || $id
            return
                if ($node/@type = "quote_add" and $node/@resp) then
                    (
                        transformKMG:transform($node/node(), $filename, $mode),
                        text { "&#8288;" },
                        element span
                        {
                            attribute class { "editorial-icon js-editorial-info--correction" },
                            attribute data-target {$id},
                            element i {
                                attribute class {"fas fa-pen"}
                            }
                        },
                        element div {
                            attribute id {$id},
                            attribute class {"modal-body__content"},
                            "Hinzufügung von",
                            transformKMG:resolve-name($node),
                            let $id := substring-after(transformKMG:resolve-reference($node), ":")
                            return
                                element span
                                {
                                    attribute class { "editorial-icon js-editorial-info--person" },
                                    attribute data-target {$id},
                                    element i {
                                        attribute class {"fas fa-user"}
                                    }
                                },
                            "."
                        }
                    )
                else if ( $node/@type = "margin" and (not($node/ancestor::tei:rdg[@type = "varying-structure" ]) and $mode = "digital") ) then
                    element aside { transformKMG:transform($node/node(), $filename, $mode) }
                else
                    transformKMG:transform($node/node(), $filename, $mode)

        case element ( tei:table )
        return
            element div
            {
                attribute class { "table" },
                element table {
                    transformKMG:transform($node/node(), $filename, $mode)
                },
                if ( $node//tei:note ) then
                    for $note in $node//tei:note[not(@type)] return
                        if ( $note/parent::tei:rdg and not(contains($note/parent::tei:rdg/@wit, $mode)) ) then
                            ()
                        else
                            element div {
                                attribute class {"authorial-note"},
                                transformKMG:transform($note/node(), $filename, $mode)
                            }
                else
                    ()
            }

        case element (tei:teiHeader) return
            ()

        case element (tei:text) return
            if ($mode = "commentary") then
                if ($node/@type = "commentary") then
                    transformKMG:transform($node/node(), $filename, $mode)
                else ()
            else
                if ($node/@type = "commentary") then ()
                else
                     if ( $node//tei:seg[@type="margin"] ) then
                        element div
                        {
                            attribute class { "text--margin" },
                            transformKMG:transform($node/node(), $filename, $mode)
                        }
                    else
                        transformKMG:transform($node/node(), $filename, $mode)

        default return
            transformKMG:transform($node/node(), $filename, $mode)
};

(:~
 : Creates the correct kind of quotation marks around a quote
 :
 : @param $node A TEI-node containing a quotation
 : @param $filename E.g. "1234v.xml"
 : @param $mode The edition view being transformed.
 : @return A quotation
 :)
declare function transformKMG:make-quote(
    $node as node(), $filename as xs:string, $mode)
as element(q)
{
    let $content := transformKMG:transform($node/node(), $filename, $mode)
    return
    element q
        {
            if ($node/@type) then attribute type {$node/@type} else (),
            switch ($node/@rend)
                case "double" return
                    ("&#x201E;", $content, "&#x201C;")
                case "single" return
                    ("&#x201A;", $content, "&#x2018;")
                case "qm_low" return
                    ("&#x201E;", $content, "&#x201C;")
                case "qm_high" return
                    ("&#x201C;", $content, "&#x201D;")
                case "g_out" return
                    ("&#xAB;", $content, "&#xBB;")
                case "g_in" return
                    ("&#xBB;", $content, "&#xAB;")
                default return
                    $content
    }
};

(:~
 : Combines a tei:item and the preceding tei:label into one
 : list item
 :
 : @param $node A tei:item
 : @param $filename E.g. "1234v.xml
 : @param $mode The edition view being transformed.
 : @return A list item
 :)
declare function transformKMG:make-list-item(
    $node as node(), $filename as xs:string, $mode)
as node()*
{
    let $label := $node/preceding-sibling::tei:label[1]
    return
    element li {
        attribute class { "list-item" },
        if ($node/@xml:id) then
            attribute id {$node/@xml:id}
        else (),
        if ($label) then
            element span {
                attribute class {"list-item__label"},
                if ( $label/tei:app) then
                    transformKMG:transform($label/node(), $filename, $mode)
                else
                    $label/text()
            }
        else
            (),
        transformKMG:transform($node/node(), $filename, $mode)
    }
};

(:~
 : This function transforms a tei:app. Initiates the transformation of the
 : lemma, as well as any readings.
 :
 : Keep in mind that the produced readings are moved before storage.
 :
 : @param $node A tei:app
 : @param $filename E.g. "1234v.xml"
 : @param $mode The edition view being transformed.
 : @return The apparatus, transformed for HTML-presentation
 :)
declare function transformKMG:make-interactive-apparatus(
    $node as node(), $filename as xs:string, $mode as xs:string)
as node()*
{
    if ($node/ancestor::bas) then
        ()
    else
        let $readings := $node/child::tei:rdg
        let $readings-no := count($readings)
        return
            (transformKMG:transform($node/tei:lem, $filename, $mode),
            for $reading in $readings return
                (transformKMG:make-reading($reading, $filename, $mode),
                if ($readings-no gt 1 and $reading = $readings[last()]) then
                    ()
                else
                    element div {
                        attribute class {"rdg-digital__divider"}
                    }
                )
            )
};

(:~
 : This function transforms a tei:lem.
 :
 : @param $node A tei:lem
 : @param $filename E.g. "1234v.xml"
 : @param $mode The edition view being transformed.
 : @return A lemma, transformed for HTML presentation
 :)
declare function transformKMG:make-interactive-lemma(
    $node, $filename, $mode)
as node()*
{
        let $reading := $node/following-sibling::tei:*
        let $reading-wits := string-join($reading/@wit, "") => replace("[# ]+", "") => replace(" ", "") => lower-case()
        let $reading-wits := if ( string-length($reading-wits) > 1 ) then ( "multiple" ) else ( $reading-wits )
        let $id := util:hash(generate-id($node/parent::*), "md5")
        return
            if ($node/node()) then
                if ($node//tei:div or $node//tei:p or $node//tei:lg or $node//tei:list or $node//tei:figure or $node//tei:graphic or $node//tei:note or $node/tei:item) then
                    (element div {
                        attribute class
                        {
                            "lemma lemma--block",
                            if ( $node/ancestor::tei:text/descendant::tei:seg[@type = "margin"] ) then
                                "margin"
                            else
                                ""
                        },
                        attribute id {$id},
                        if ($reading-wits) then
                            attribute wits {$reading-wits}
                        else
                            (),
                        transformKMG:transform($node/node(), $filename, $mode)
                    })
                else
                    (element span {
                        attribute class
                        {
                            "lemma",
                            if ( $node/ancestor::tei:text/descendant::tei:seg[@type = "margin"] ) then
                                "margin"
                            else
                                ""
                        },
                        attribute id {$id},
                        if ($reading-wits) then
                            attribute wits {$reading-wits}
                        else
                            (),
                        transformKMG:transform($node/node(), $filename, $mode)
                    })
            else
                element span {
                    attribute class
                    {
                        "lemma lemma--insertion",
                        if ( $node/ancestor::tei:text/descendant::tei:seg[@type = "margin"] ) then
                            "margin"
                        else
                            ""
                    },
                    attribute id {$id},
                    if ($reading-wits) then
                        attribute wits {$reading-wits}
                    else
                        (),
                    element sup {$reading-wits},
                    text {"&#x221A;"}
                }
};

(:~
 : Transforms a tei:rdg into a span containing the reading, links it
 : to the corresponding lemma.
 :
 : @param $reading A tei:rdg
 : @param $filename E.g. "1234v.xml"
 : @param $mode The edition view being transformed.
 : @return A sequence of span
 :)
declare function transformKMG:make-reading(
    $reading as node(), $filename as xs:string, $mode)
as element()*
{
    let $wits := replace($reading/@wit, "[#]+", "")
    let $wits := replace($wits, " ", "") => lower-case()
    let $wits-attribute := if ( string-length($wits) > 1 ) then ( "multiple" ) else ( $wits )

    let $reading-id := util:hash(generate-id($reading), "md5")
    let $lemma-id := util:hash(generate-id($reading/parent::*), "md5")
    let $target-lemma := "#" || $lemma-id
    let $target-reading := "#" || $reading-id
    return
        element span {
            attribute class
            {
                "rdg-digital rdg-digital--" || $wits-attribute,
                if ( $reading/@type = "varying-structure" ) then
                    " varying-structure"
                else
                    ""
            },
            attribute id {$reading-id},
            attribute target-lem {$target-lemma},
            attribute wits {$wits-attribute},
            element table {
                attribute class { "rdg-digital-table" },
                element tr {
                    if ( $reading//tei:seg[@type="margin"] ) then
                        attribute class { "paragraph--margin" }
                    else
                        (),
                    element td {
                        attribute class {"rdg-digital-table__siglum"},
                        element strong {$wits}
                    },
                    element td {
                        attribute class {"rdg-digital-table__content"},
                        if ($reading/node()) then
                            transformKMG:transform($reading/node(), $filename, $mode)
                        else
                            element i {"&#xF8;"}
                    }
                }
            }
        }
};

(:~
 : Resolves an abbreviated reference, adding a base-url to an id.
 :
 : @param $node A node containing an @corresp, @ref, @target, or @resp
 : @return A string containing the full reference
 :)

declare function transformKMG:resolve-reference($node as node())
as xs:string?
{
    $node/root()//*[@xml:id = replace($node/attribute()[name() = ('corresp', 'ref', 'target', 'resp')], '#', '')]/@ref
};

(:~
 : Retrieves a name that is referenced elsewhere in the document.l
 :
 : @param $node A node containing an @corresp, @ref, @target, or @resp
 : @return A string containing the resolved name
 :)

declare function transformKMG:resolve-name($node as node())
as xs:string?
{
    string-join(
        $node/root()//*[@xml:id = replace($node/attribute()[name() = ('corresp', 'ref', 'target', 'resp')], '#', '')]/text()
  )
};

(:~
 : Retrieves the target node of a given @copyOf-reference.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @param $copyOf A reference to another node
 : @return The node the @copyOf refers to
 :)
declare function transformKMG:find-corresp-node(
    $node as node(), $copyOf as xs:string)
as node()?
{
    if (fn:starts-with($copyOf, "textgrid:"))
    then
        let $uri := substring($copyOf, 10, 5)
        let $doc := doc($config:sade-data || $uri || '.xml')
        let $copyOf := substring-after($copyOf, "#")
        return
            $doc//*[@xml:id = $copyOf]
    else
        let $copyOf := replace($copyOf, "#", "")
        return
            $node/root()//*[@xml:id = $copyOf]
};

(:~
 : This function resolves a tei:join according to its attributes result, scope,
 : and target.
 :
 : @param $node A tei:join
 : @return A constructed node, according to the specifications of the tei:join
 :)
declare function transformKMG:resolve-join($node as node())
as node()?
{
    let $result := "tei:" || $node/@result
    let $attributes := $node/@*[local-name(.) != "result" or "scope" or "target"]
    let $attributes :=
        for $attribute in $attributes return
            attribute {local-name($attribute)} {$attribute}
    let $targets := tokenize($node/@target)
    return
        element {$result} {
            $attributes,
            for $target in $targets return
                for $target in $targets return
                if (fn:starts-with($target, "textgrid:")) then
                    let $uri := substring($target, 10, 5)
                    let $doc := doc($config:sade-data || $uri || '.xml')
                    let $target := substring-after($target, "#")
                    return
                        if ( $node/@scope = "branches" ) then
                            $doc//*[@xml:id = $target]/node()
                        else
                            $doc//*[@xml:id = $target]
                else
                    let $target := replace($target, "#", "")
                    return
                        if ( $node/@scope = "branches" ) then
                            $node/ancestor::tei:app//*[@xml:id = $target]/node()
                        else
                            $node/ancestor::tei:app//*[@xml:id = $target]
        }
};

(:~
 : This function copies a TEI-node.
 :
 : @param $node A TEI-node with an attribute copyOf referencing another node
 : @return The desired copy as a constructed node
 :)
declare function local:copy-node($node, $filename, $mode)
as node()*
{
    let $target := transformKMG:find-corresp-node($node, $node/@copyOf)
    let $attributes := $node/@*[local-name(.) != "copyOf"]
    let $attributes :=
        for $attribute in $attributes return
            attribute {local-name($attribute)} {$attribute}
    let $mode :=
        if ( $node/ancestor::tei:rdg[@type = "varying-structure" ] ) then
            $node/ancestor::tei:rdg[@type = "varying-structure" ]/@wit => replace("#", "")
        else
            $mode
    let $result :=
        element {node-name($node)} {
            $attributes,
            $target/node()
        }
    return
        transformKMG:transform($result, $filename, $mode)
};

(:~
 : This function transforms tei:media into the corresponding html5 media-element
 :
 : @TODO: adapt source to final location of media objects
 : @param $node A tei:media
 : @return A html5 media-element
 :)
declare function transformKMG:media($node as node()*)
as node()*
{
    element { substring-before($node/@mimeType, "/") }
    {
        attribute controls { () },
        element source
        {
            attribute type { $node/@mimeType },
            $node/@url
        }
    },
    element span { $node/tei:desc/text() }
};

declare function transformKMG:choice($node, $filename, $mode)
{
    let $id := util:hash(generate-id($node/*[1]), "md5")
    let $target := "#" || $id
    let $text := element span {
            if ($node/tei:sic/node()) then
                transformKMG:transform($node/tei:sic, $filename, $mode)
            else if ( $node/tei:abbr ) then
                transformKMG:transform($node/tei:abbr, $filename, $mode)
            else
                element i {"&#xF8;"},
            element span {
                attribute class
                {
                    "editorial-icon",
                    if ( $node/tei:sic ) then
                        "js-editorial-info--corr correction"
                    else
                        "js-editorial-info--abbr abbreviation"
                },
                attribute data-target {$id},
                element i {
                    attribute class {"fas fa-pen"}
                }
            }
        }
    let $modalcontent :=    element div {
            attribute id {$id},
            attribute class {"modal-body__content"},
            if ( $node/tei:corr ) then
                (transformKMG:transform($node/tei:corr, $filename, $mode),
                " ["
                || transformKMG:resolve-name($node/tei:corr),
                if ($node/tei:corr/@resp) then
                    let $id := substring-after(transformKMG:resolve-reference($node/tei:corr), ":")
                        return
                            element span
                            {
                                attribute class { "editorial-icon js-editorial-info--person" },
                                attribute data-target {$id},
                                element i {
                                    attribute class {"fas fa-user"}
                                }
                            }
                else
                    (),
                "]")
            else
                transformKMG:transform($node/tei:expan, $filename, $mode)
        }
    return
        ($modalcontent, $text)
};

declare function transformKMG:getHiRendStyleFromHeader($filename as xs:string, $witnessID as xs:string) as xs:string{
    let $doc := doc($config:sade-data || $filename)
    let $fileDesc :=
        if (fn:exists($doc//tei:teiHeader/tei:fileDesc/tei:sourceDesc/tei:listWit/tei:witness[@xml:id=$witnessID])) then (
            $doc//tei:teiHeader/tei:fileDesc/tei:sourceDesc/tei:listWit/tei:witness[@xml:id=$witnessID]/@rend/string()
            )
        else ""
    return $fileDesc
};
